@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -29,10 +29,11 @@
 #define HAVE_PREAD_PWRITE 1
 
 
-#include <assert.h>
 #include <unistd.h>
 #include <sys/types.h>
 
+#include "logging.h"
+
 
 /* Functions in files.c */
 
@@ -40,108 +41,51 @@ char *FileName(const char *dirname,const char *prefix, const char *name);
 
 void *MapFile(const char *filename);
 void *MapFileWriteable(const char *filename);
-void *UnmapFile(const char *filename);
 
-int OpenFileNew(const char *filename);
-int OpenFileAppend(const char *filename);
-int ReOpenFile(const char *filename);
-int ReOpenFileWriteable(const char *filename);
+void *UnmapFile(const void *address);
 
-static int WriteFile(int fd,const void *address,size_t length);
-static int ReadFile(int fd,void *address,size_t length);
+int SlimMapFile(const char *filename);
+int SlimMapFileWriteable(const char *filename);
 
-static int SeekWriteFile(int fd,const void *address,size_t length,off_t position);
-static int SeekReadFile(int fd,void *address,size_t length,off_t position);
+int SlimUnmapFile(int fd);
+
+int OpenFileBufferedNew(const char *filename);
+int OpenFileBufferedAppend(const char *filename);
+
+int ReOpenFileBuffered(const char *filename);
+
+int WriteFileBuffered(int fd,const void *address,size_t length);
+int ReadFileBuffered(int fd,void *address,size_t length);
+
+int SeekFileBuffered(int fd,off_t position);
+int SkipFileBuffered(int fd,off_t skip);
+
+int CloseFileBuffered(int fd);
+
+int OpenFile(const char *filename);
+
+void CloseFile(int fd);
 
 off_t SizeFile(const char *filename);
+off_t SizeFileFD(int fd);
 int ExistsFile(const char *filename);
 
-static int SeekFile(int fd,off_t position);
+int DeleteFile(const char *filename);
 
-int CloseFile(int fd);
+int RenameFile(const char *oldfilename,const char *newfilename);
 
-int DeleteFile(char *filename);
+/* Functions in files.h */
+
+static inline int SlimReplace(int fd,const void *address,size_t length,off_t position);
+static inline int SlimFetch(int fd,void *address,size_t length,off_t position);
 
 
 /* Inline the frequently called functions */
 
 /*++++++++++++++++++++++++++++++++++++++
-  Write data to a file descriptor.
+  Write data to a file that has been opened for slim mode access.
 
-  int WriteFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor to write to.
-
-  const void *address The address of the data to be written.
-
-  size_t length The length of data to write.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static inline int WriteFile(int fd,const void *address,size_t length)
-{
- assert(fd!=-1);
-
- /* Write the data */
-
- if(write(fd,address,length)!=length)
-    return(-1);
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Read data from a file descriptor.
-
-  int ReadFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor to read from.
-
-  void *address The address the data is to be read into.
-
-  size_t length The length of data to read.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static inline int ReadFile(int fd,void *address,size_t length)
-{
- assert(fd!=-1);
-
- /* Read the data */
-
- if(read(fd,address,length)!=length)
-    return(-1);
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Seek to a position in a file descriptor.
-
-  int SeekFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor to seek within.
-
-  off_t position The position to seek to.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static inline int SeekFile(int fd,off_t position)
-{
- assert(fd!=-1);
-
- /* Seek the data */
-
- if(lseek(fd,position,SEEK_SET)!=position)
-    return(-1);
-
- return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Write data to a file descriptor after seeking to a position.
-
-  int WriteFile Returns 0 if OK or something else in case of an error.
+  int SlimReplace Returns 0 if OK or something else in case of an error.
 
   int fd The file descriptor to write to.
 
@@ -149,18 +93,16 @@ static inline int SeekFile(int fd,off_t position)
 
   size_t length The length of data to write.
 
-  off_t position The position to seek to.
+  off_t position The position in the file to seek to.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static inline int SeekWriteFile(int fd,const void *address,size_t length,off_t position)
+static inline int SlimReplace(int fd,const void *address,size_t length,off_t position)
 {
- assert(fd!=-1);
-
  /* Seek and write the data */
 
 #if HAVE_PREAD_PWRITE
 
- if(pwrite(fd,address,length,position)!=length)
+ if(pwrite(fd,address,length,position)!=(ssize_t)length)
     return(-1);
 
 #else
@@ -178,9 +120,9 @@ static inline int SeekWriteFile(int fd,const void *address,size_t length,off_t p
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Read data from a file descriptor after seeking to a position.
+  Read data from a file that has been opened for slim mode access.
 
-  int SeekReadFile Returns 0 if OK or something else in case of an error.
+  int SlimFetch Returns 0 if OK or something else in case of an error.
 
   int fd The file descriptor to read from.
 
@@ -188,18 +130,16 @@ static inline int SeekWriteFile(int fd,const void *address,size_t length,off_t p
 
   size_t length The length of data to read.
 
-  off_t position The position to seek to.
+  off_t position The position in the file to seek to.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static inline int SeekReadFile(int fd,void *address,size_t length,off_t position)
+static inline int SlimFetch(int fd,void *address,size_t length,off_t position)
 {
- assert(fd!=-1);
-
  /* Seek and read the data */
 
 #if HAVE_PREAD_PWRITE
 
- if(pread(fd,address,length,position)!=length)
+ if(pread(fd,address,length,position)!=(ssize_t)length)
     return(-1);
 
 #else
