@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -26,15 +26,36 @@
 #include <stdio.h>
 #include <sys/time.h>
 
+#include "typesx.h"
+
+
+/* Data structures */
+
+/*+ A structure containing a single object as written by the logerror_*() functions. +*/
+typedef struct _ErrorLogObject
+{
+ char      type;             /*+ The type of the object. +*/
+
+ uint64_t  id;               /*+ The id of the object. +*/
+
+ uint32_t  offset;           /*+ The offset of the error message from the beginning of the text file. +*/
+}
+ ErrorLogObject;
+
 
 /* Variables */
 
 extern int option_loggable;
 extern int option_logtime;
+extern int option_logmemory;
 
 
 /* Runtime progress logging functions in logging.c */
 
+void printf_program_start(void);
+void printf_program_end(void);
+
+
 #ifdef __GNUC__
 
 void printf_first(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
@@ -57,11 +78,16 @@ void fprintf_last(FILE *file,const char *format, ...);
 
 #endif
 
-void fprintf_elapsed_time(FILE *file,struct timeval *start);
+void log_malloc(void *address,size_t size);
+void log_free(void *address);
 
-/* Parsing/processing error logging functions in logging.c */
+void log_mmap(size_t size);
+void log_munmap(size_t size);
 
-void open_errorlog(const char *filename,int append);
+
+/* Error logging functions in logerror.c */
+
+void open_errorlog(const char *filename,int append,int bin);
 void close_errorlog(void);
 
 #ifdef __GNUC__
@@ -74,5 +100,16 @@ void logerror(const char *format, ...);
 
 #endif
 
+node_t     logerror_node    (node_t     id);
+way_t      logerror_way     (way_t      id);
+relation_t logerror_relation(relation_t id);
+
+
+/* Runtime fatal error assertion in logging.c */
+
+#define logassert(xx,yy) do { if(!(xx)) _logassert(yy,__FILE__,__LINE__); } while(0)
+
+void _logassert(const char *message,const char *file,int line);
+
 
 #endif /* LOGGING_H */
