@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -52,6 +52,7 @@ static latlong_t lat_min,lat_max,lon_min,lon_max;
 static int sort_by_id(NodeX *a,NodeX *b);
 static int deduplicate_and_index_by_id(NodeX *nodex,index_t index);
 
+static int update_id(NodeX *nodex,index_t index);
 static int sort_by_lat_long(NodeX *a,NodeX *b);
 static int index_by_lat_long(NodeX *nodex,index_t index);
 
@@ -93,12 +94,17 @@ NodesX *NewNodeList(int append,int readonly)
       }
 
  if(append)
-    nodesx->fd=OpenFileAppend(nodesx->filename_tmp);
+    nodesx->fd=OpenFileBufferedAppend(nodesx->filename_tmp);
  else if(!readonly)
-    nodesx->fd=OpenFileNew(nodesx->filename_tmp);
+    nodesx->fd=OpenFileBufferedNew(nodesx->filename_tmp);
  else
     nodesx->fd=-1;
 
+#if SLIM
+ nodesx->cache=NewNodeXCache();
+ log_malloc(nodesx->cache,sizeof(*nodesx->cache));
+#endif
+
  return(nodesx);
 }
 
@@ -122,16 +128,33 @@ void FreeNodeList(NodesX *nodesx,int keep)
  free(nodesx->filename_tmp);
 
  if(nodesx->idata)
+   {
+    log_free(nodesx->idata);
     free(nodesx->idata);
+   }
 
  if(nodesx->gdata)
+   {
+    log_free(nodesx->gdata);
     free(nodesx->gdata);
+   }
 
  if(nodesx->pdata)
+   {
+    log_free(nodesx->pdata);
     free(nodesx->pdata);
+   }
 
  if(nodesx->super)
+   {
+    log_free(nodesx->super);
     free(nodesx->super);
+   }
+
+#if SLIM
+ log_free(nodesx->cache);
+ DeleteNodeXCache(nodesx->cache);
+#endif
 
  free(nodesx);
 }
@@ -163,7 +186,7 @@ void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,tr
  nodex.allow=allow;
  nodex.flags=flags;
 
- WriteFile(nodesx->fd,&nodex,sizeof(NodeX));
+ WriteFileBuffered(nodesx->fd,&nodex,sizeof(NodeX));
 
  nodesx->number++;
 
@@ -180,7 +203,7 @@ void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,tr
 void FinishNodeList(NodesX *nodesx)
 {
  if(nodesx->fd!=-1)
-    nodesx->fd=CloseFile(nodesx->fd);
+    nodesx->fd=CloseFileBuffered(nodesx->fd);
 }
 
 
@@ -200,6 +223,15 @@ index_t IndexNodeX(NodesX *nodesx,node_t id)
  index_t end=nodesx->number-1;
  index_t mid;
 
+ if(nodesx->number==0)          /* No nodes */
+    return(NO_NODE);
+
+ if(id<nodesx->idata[start])    /* Key is before start */
+    return(NO_NODE);
+
+ if(id>nodesx->idata[end])      /* Key is after end */
+    return(NO_NODE);
+
  /* Binary search - search key exact match only is required.
   *
   *  # <- start  |  Check mid and move start or end if it doesn't match
@@ -211,33 +243,24 @@ index_t IndexNodeX(NodesX *nodesx,node_t id)
   *  # <- end    |  start or end is the wanted one.
   */
 
- if(end<start)                        /* There are no nodes */
-    return(NO_NODE);
- else if(id<nodesx->idata[start])     /* Check key is not before start */
-    return(NO_NODE);
- else if(id>nodesx->idata[end])       /* Check key is not after end */
-    return(NO_NODE);
- else
+ do
    {
-    do
-      {
-       mid=(start+end)/2;             /* Choose mid point */
+    mid=(start+end)/2;             /* Choose mid point */
 
-       if(nodesx->idata[mid]<id)      /* Mid point is too low */
-          start=mid+1;
-       else if(nodesx->idata[mid]>id) /* Mid point is too high */
-          end=mid?(mid-1):mid;
-       else                           /* Mid point is correct */
-          return(mid);
-      }
-    while((end-start)>1);
-
-    if(nodesx->idata[start]==id)      /* Start is correct */
-       return(start);
-
-    if(nodesx->idata[end]==id)        /* End is correct */
-       return(end);
+    if(nodesx->idata[mid]<id)      /* Mid point is too low */
+       start=mid+1;
+    else if(nodesx->idata[mid]>id) /* Mid point is too high */
+       end=mid?(mid-1):mid;
+    else                           /* Mid point is correct */
+       return(mid);
    }
+ while((end-start)>1);
+
+ if(nodesx->idata[start]==id)      /* Start is correct */
+    return(start);
+
+ if(nodesx->idata[end]==id)        /* End is correct */
+    return(end);
 
  return(NO_NODE);
 }
@@ -260,15 +283,16 @@ void SortNodeList(NodesX *nodesx)
 
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ nodesx->fd=ReOpenFileBuffered(nodesx->filename_tmp);
 
  DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename_tmp);
+ fd=OpenFileBufferedNew(nodesx->filename_tmp);
 
  /* Allocate the array of indexes */
 
  nodesx->idata=(node_t*)malloc(nodesx->number*sizeof(node_t));
+ log_malloc(nodesx->idata,nodesx->number*sizeof(node_t));
 
  logassert(nodesx->idata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
@@ -282,10 +306,12 @@ void SortNodeList(NodesX *nodesx)
                                                            (int (*)(const void*,const void*))sort_by_id,
                                                            (int (*)(void*,index_t))deduplicate_and_index_by_id);
 
+ nodesx->knumber=nodesx->number;
+
  /* Close the files */
 
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
+ nodesx->fd=CloseFileBuffered(nodesx->fd);
+ CloseFileBuffered(fd);
 
  /* Print the final message */
 
@@ -354,44 +380,118 @@ static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
 
   NodesX *nodesx The set of nodes to modify.
 
-  SegmentsX *segmentsx The set of segments to use.
+  WaysX *waysx The set of ways to use.
 
   int keep If set to 1 then keep the old data file otherwise delete it.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,int keep)
+void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep)
 {
+ BitMask *usednode;
  NodeX nodex;
- index_t total=0,highway=0,nothighway=0;
+ index_t i,total=0,highway=0,nothighway=0;
  int fd;
 
  /* Print the start message */
 
- printf_first("Checking Nodes: Nodes=0");
+ printf_first("Checking Ways for unused Nodes: Ways=0 Highway Nodes=0");
+
+ /* Allocate the node usage bitmask */
+
+ usednode=AllocBitMask(nodesx->number);
+ log_malloc(usednode,LengthBitMask(nodesx->number)*sizeof(BitMask));
+
+ logassert(usednode,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
+
+ /* Re-open the file read-only */
+
+ waysx->fd=ReOpenFileBuffered(waysx->filename_tmp);
+
+ /* Loop through the ways and mark the used nodes */
+
+ for(i=0;i<waysx->number;i++)
+   {
+    WayX wayx;
+    FILESORT_VARINT waysize;
+    node_t node;
+
+    ReadFileBuffered(waysx->fd,&waysize,FILESORT_VARSIZE);
+
+    ReadFileBuffered(waysx->fd,&wayx,sizeof(WayX));
+
+    while(!ReadFileBuffered(waysx->fd,&node,sizeof(node_t)) && node!=NO_NODE_ID)
+      {
+       index_t index=IndexNodeX(nodesx,node);
+
+       waysize-=sizeof(node_t);
+
+       if(index!=NO_NODE)
+         {
+          if(!IsBitSet(usednode,index))
+             highway++;
+
+          SetBit(usednode,index);
+         }
+      }
+
+    waysize-=sizeof(node_t)+sizeof(WayX);
+
+    SkipFileBuffered(waysx->fd,waysize);
+
+    if(!((i+1)%1000))
+       printf_middle("Checking Ways for unused Nodes: Ways=%"Pindex_t" Highway Nodes=%"Pindex_t,i+1,highway);
+   }
+
+ /* Free the now-unneeded index */
+
+ log_free(nodesx->idata);
+ free(nodesx->idata);
+ nodesx->idata=NULL;
+
+ /* Close the file */
+
+ waysx->fd=CloseFileBuffered(waysx->fd);
+
+ /* Print the final message */
+
+ printf_last("Checked Ways for unused Nodes: Ways=%"Pindex_t" Highway Nodes=%"Pindex_t,waysx->number,highway);
+
+
+ /* Print the start message */
+
+ printf_first("Removing unused Nodes: Nodes=0");
+
+ /* Allocate the array of indexes */
+
+ nodesx->idata=(node_t*)malloc(highway*sizeof(node_t));
+ log_malloc(nodesx->idata,highway*sizeof(node_t));
+
+ logassert(nodesx->idata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+
+ highway=0;
 
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ nodesx->fd=ReOpenFileBuffered(nodesx->filename_tmp);
 
  if(keep)
     RenameFile(nodesx->filename_tmp,nodesx->filename);
  else
     DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename_tmp);
+ fd=OpenFileBufferedNew(nodesx->filename_tmp);
 
  /* Modify the on-disk image */
 
- while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
+ while(!ReadFileBuffered(nodesx->fd,&nodex,sizeof(NodeX)))
    {
-    if(!IsBitSet(segmentsx->usednode,total))
+    if(!IsBitSet(usednode,total))
        nothighway++;
     else
       {
-       nodex.id=highway;
-       nodesx->idata[highway]=nodesx->idata[total];
+       nodesx->idata[highway]=nodex.id;
 
-       WriteFile(fd,&nodex,sizeof(NodeX));
+       WriteFileBuffered(fd,&nodex,sizeof(NodeX));
 
        highway++;
       }
@@ -399,24 +499,24 @@ void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,int keep)
     total++;
 
     if(!(total%10000))
-       printf_middle("Checking Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
+       printf_middle("Removing unused Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
    }
 
  nodesx->number=highway;
 
  /* Close the files */
 
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
+ nodesx->fd=CloseFileBuffered(nodesx->fd);
+ CloseFileBuffered(fd);
 
  /* Free the now-unneeded index */
 
- free(segmentsx->usednode);
- segmentsx->usednode=NULL;
+ log_free(usednode);
+ free(usednode);
 
  /* Print the final message */
 
- printf_last("Checked Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
+ printf_last("Removed unused Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
 }
 
 
@@ -434,6 +534,9 @@ void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx)
  index_t total=0,pruned=0,notpruned=0;
  int fd;
 
+ if(nodesx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Deleting Pruned Nodes: Nodes=0 Pruned=0");
@@ -441,20 +544,21 @@ void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx)
  /* Allocate the array of indexes */
 
  nodesx->pdata=(index_t*)malloc(nodesx->number*sizeof(index_t));
+ log_malloc(nodesx->pdata,nodesx->number*sizeof(index_t));
 
  logassert(nodesx->pdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ nodesx->fd=ReOpenFileBuffered(nodesx->filename_tmp);
 
  DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename_tmp);
+ fd=OpenFileBufferedNew(nodesx->filename_tmp);
 
  /* Modify the on-disk image */
 
- while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
+ while(!ReadFileBuffered(nodesx->fd,&nodex,sizeof(NodeX)))
    {
     if(segmentsx->firstnode[total]==NO_SEGMENT)
       {
@@ -464,10 +568,9 @@ void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx)
       }
     else
       {
-       nodex.id=notpruned;
        nodesx->pdata[total]=notpruned;
 
-       WriteFile(fd,&nodex,sizeof(NodeX));
+       WriteFileBuffered(fd,&nodex,sizeof(NodeX));
 
        notpruned++;
       }
@@ -482,8 +585,17 @@ void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx)
 
  /* Close the files */
 
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
+ nodesx->fd=CloseFileBuffered(nodesx->fd);
+ CloseFileBuffered(fd);
+
+ /* Free the no-longer required memory */
+
+ if(segmentsx->firstnode)
+   {
+    log_free(segmentsx->firstnode);
+    free(segmentsx->firstnode);
+    segmentsx->firstnode=NULL;
+   }
 
  /* Print the final message */
 
@@ -502,61 +614,72 @@ void SortNodeListGeographically(NodesX *nodesx)
  int fd;
  ll_bin_t lat_min_bin,lat_max_bin,lon_min_bin,lon_max_bin;
 
- /* While we are here we can work out the range of data */
+ if(nodesx->number==0)
+    return;
+
+ /* Print the start message */
+
+ printf_first("Sorting Nodes Geographically");
+
+ /* Work out the range of data */
 
  lat_min=radians_to_latlong( 2);
  lat_max=radians_to_latlong(-2);
  lon_min=radians_to_latlong( 4);
  lon_max=radians_to_latlong(-4);
 
- /* Print the start message */
-
- printf_first("Sorting Nodes Geographically");
-
  /* Allocate the memory for the geographical index array */
 
  nodesx->gdata=(index_t*)malloc(nodesx->number*sizeof(index_t));
+ log_malloc(nodesx->gdata,nodesx->number*sizeof(index_t));
 
  logassert(nodesx->gdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ nodesx->fd=ReOpenFileBuffered(nodesx->filename_tmp);
 
  DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename_tmp);
+ fd=OpenFileBufferedNew(nodesx->filename_tmp);
 
  /* Sort nodes geographically and index them */
 
  sortnodesx=nodesx;
 
- filesort_fixed(nodesx->fd,fd,sizeof(NodeX),NULL,
+ filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(void*,index_t))update_id,
                                             (int (*)(const void*,const void*))sort_by_lat_long,
                                             (int (*)(void*,index_t))index_by_lat_long);
 
  /* Close the files */
 
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
-
- /* Free the memory */
-
- free(nodesx->super);
- nodesx->super=NULL;
+ nodesx->fd=CloseFileBuffered(nodesx->fd);
+ CloseFileBuffered(fd);
 
  /* Work out the number of bins */
 
- lat_min_bin=latlong_to_bin(lat_min);
- lon_min_bin=latlong_to_bin(lon_min);
- lat_max_bin=latlong_to_bin(lat_max);
- lon_max_bin=latlong_to_bin(lon_max);
+ if(nodesx->super)
+   {
+    lat_min_bin=latlong_to_bin(lat_min);
+    lon_min_bin=latlong_to_bin(lon_min);
+    lat_max_bin=latlong_to_bin(lat_max);
+    lon_max_bin=latlong_to_bin(lon_max);
 
- nodesx->latzero=lat_min_bin;
- nodesx->lonzero=lon_min_bin;
+    nodesx->latzero=lat_min_bin;
+    nodesx->lonzero=lon_min_bin;
 
- nodesx->latbins=(lat_max_bin-lat_min_bin)+1;
- nodesx->lonbins=(lon_max_bin-lon_min_bin)+1;
+    nodesx->latbins=(lat_max_bin-lat_min_bin)+1;
+    nodesx->lonbins=(lon_max_bin-lon_min_bin)+1;
+   }
+
+ /* Free the memory */
+
+ if(nodesx->super)
+   {
+    log_free(nodesx->super);
+    free(nodesx->super);
+    nodesx->super=NULL;
+   }
 
  /* Print the final message */
 
@@ -564,6 +687,27 @@ void SortNodeListGeographically(NodesX *nodesx)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Update the node ids.
+
+  int update_id Return 1 if the value is to be kept, otherwise 0.
+
+  NodeX *nodex The extended node.
+
+  index_t index The number of unsorted nodes that have been read from the input file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int update_id(NodeX *nodex,index_t index)
+{
+ nodex->id=index;
+
+ if(sortnodesx->super && IsBitSet(sortnodesx->super,index))
+    nodex->flags|=NODE_SUPER;
+
+ return(1);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Sort the nodes into latitude and longitude order (first by longitude bin
   number, then by latitude bin number and then by exact longitude and then by
@@ -628,17 +772,17 @@ static int index_by_lat_long(NodeX *nodex,index_t index)
 {
  sortnodesx->gdata[nodex->id]=index;
 
- if(IsBitSet(sortnodesx->super,nodex->id))
-    nodex->flags|=NODE_SUPER;
-
- if(nodex->latitude<lat_min)
-    lat_min=nodex->latitude;
- if(nodex->latitude>lat_max)
-    lat_max=nodex->latitude;
- if(nodex->longitude<lon_min)
-    lon_min=nodex->longitude;
- if(nodex->longitude>lon_max)
-    lon_max=nodex->longitude;
+ if(sortnodesx->super)
+   {
+    if(nodex->latitude<lat_min)
+       lat_min=nodex->latitude;
+    if(nodex->latitude>lat_max)
+       lat_max=nodex->latitude;
+    if(nodex->longitude<lon_min)
+       lon_min=nodex->longitude;
+    if(nodex->longitude>lon_max)
+       lon_max=nodex->longitude;
+   }
 
  return(1);
 }
@@ -677,13 +821,13 @@ void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
 
  /* Re-open the file */
 
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ nodesx->fd=ReOpenFileBuffered(nodesx->filename_tmp);
 
  /* Write out the nodes data */
 
- fd=OpenFileNew(filename);
+ fd=OpenFileBufferedNew(filename);
 
- SeekFile(fd,sizeof(NodesFile)+(nodesx->latbins*nodesx->lonbins+1)*sizeof(index_t));
+ SeekFileBuffered(fd,sizeof(NodesFile)+(nodesx->latbins*nodesx->lonbins+1)*sizeof(index_t));
 
  for(i=0;i<nodesx->number;i++)
    {
@@ -692,13 +836,13 @@ void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
     ll_bin_t latbin,lonbin;
     ll_bin2_t llbin;
 
-    ReadFile(nodesx->fd,&nodex,sizeof(NodeX));
+    ReadFileBuffered(nodesx->fd,&nodex,sizeof(NodeX));
 
     /* Create the Node */
 
     node.latoffset=latlong_to_off(nodex.latitude);
     node.lonoffset=latlong_to_off(nodex.longitude);
-    node.firstseg=segmentsx->firstnode[nodesx->gdata[nodex.id]];
+    node.firstseg=segmentsx->firstnode[i];
     node.allow=nodex.allow;
     node.flags=nodex.flags;
 
@@ -716,7 +860,7 @@ void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
 
     /* Write the data */
 
-    WriteFile(fd,&node,sizeof(Node));
+    WriteFileBuffered(fd,&node,sizeof(Node));
 
     if(!((i+1)%10000))
        printf_middle("Writing Nodes: Nodes=%"Pindex_t,i+1);
@@ -724,7 +868,7 @@ void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
 
  /* Close the file */
 
- nodesx->fd=CloseFile(nodesx->fd);
+ nodesx->fd=CloseFileBuffered(nodesx->fd);
 
  /* Finish off the offset indexing and write them out */
 
@@ -733,8 +877,8 @@ void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
  for(;latlonbin<=maxlatlonbins;latlonbin++)
     offsets[latlonbin]=nodesx->number;
 
- SeekFile(fd,sizeof(NodesFile));
- WriteFile(fd,offsets,(nodesx->latbins*nodesx->lonbins+1)*sizeof(index_t));
+ SeekFileBuffered(fd,sizeof(NodesFile));
+ WriteFileBuffered(fd,offsets,(nodesx->latbins*nodesx->lonbins+1)*sizeof(index_t));
 
  free(offsets);
 
@@ -749,10 +893,16 @@ void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
  nodesfile.latzero=nodesx->latzero;
  nodesfile.lonzero=nodesx->lonzero;
 
- SeekFile(fd,0);
- WriteFile(fd,&nodesfile,sizeof(NodesFile));
+ SeekFileBuffered(fd,0);
+ WriteFileBuffered(fd,&nodesfile,sizeof(NodesFile));
 
- CloseFile(fd);
+ CloseFileBuffered(fd);
+
+ /* Free the memory in the segments */
+
+ log_free(segmentsx->firstnode);
+ free(segmentsx->firstnode);
+ segmentsx->firstnode=NULL;
 
  /* Print the final message */
 
